@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22361 $
+ * $Author: rjongbloed $
+ * $Date: 2009-04-01 07:06:50 +0000 (Wed, 01 Apr 2009) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -278,6 +278,9 @@ class H323EndPoint : public OpalRTPEndPoint
        If the endpoint is already registered with a gatekeeper that meets
        the same criteria then the gatekeeper is not changed, otherwise it is
        deleted (with unregistration) and new one created and registered to.
+
+       Note that a gatekeeper address of "*" is treated like an empty string
+       resulting in gatekeeper discovery.
      */
     PBoolean UseGatekeeper(
       const PString & address = PString::Empty(),     ///<  Address of gatekeeper to use.
@@ -996,26 +999,6 @@ class H323EndPoint : public OpalRTPEndPoint
       */
     virtual void OnReceivedInitiateReturnError();
 
-#if OPAL_VIDEO
-    /**See if should auto-start receive video channels on connection.
-     */
-    PBoolean CanAutoStartReceiveVideo() const { return manager.CanAutoStartReceiveVideo(); }
-
-    /**See if should auto-start transmit video channels on connection.
-     */
-    PBoolean CanAutoStartTransmitVideo() const { return manager.CanAutoStartTransmitVideo(); }
-#endif
-
-#if OPAL_T38_CAPABILITY
-    /**See if should auto-start receive fax channels on connection.
-     */
-    PBoolean CanAutoStartReceiveFax() const { return autoStartReceiveFax; }
-
-    /**See if should auto-start transmit fax channels on connection.
-     */
-    PBoolean CanAutoStartTransmitFax() const { return autoStartTransmitFax; }
-#endif
-
     /**See if should automatically do call forward of connection.
      */
     PBoolean CanAutoCallForward() const { return autoCallForward; }
@@ -1346,9 +1329,6 @@ class H323EndPoint : public OpalRTPEndPoint
     // Configuration variables, commonly changed
     PStringList     localAliasNames;
     PStringList     localAliasPatterns;
-    PBoolean        autoStartReceiveFax;
-    PBoolean        autoStartTransmitFax;
-    PBoolean        isH224Enabled;
     PBoolean        autoCallForward;
     PBoolean        disableFastStart;
     PBoolean        disableH245Tunneling;
