@@ -35,9 +35,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 21293 $
+ * $Revision: 22680 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2009-05-21 00:48:57 +0000 (Thu, 21 May 2009) $
 */
 
 #ifndef OPAL_H460_H4601_H
@@ -1033,7 +1033,7 @@ class H460_FeatureSet : public PObject
 
     /** Build a FeatureSet from the contents of a generic data field.
 	 */
-	H460_FeatureSet(const H225_ArrayOf_GenericData & generic);
+	H460_FeatureSet(const H225_ArrayOf_GenericData & genericData);
 
     /** Derive new Feature Set based on this Feature Set ie Clone this FeatureSet
 	  */
