@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21833 $
+ * $Revision: 22096 $
  * $Author: rjongbloed $
- * $Date: 2008-12-18 05:15:31 +0000 (Thu, 18 Dec 2008) $
+ * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_MANAGER_H
@@ -378,6 +378,7 @@ class OpalManager : public PObject
        variable and uses MakeConnection() to start the B-party connection.
       */
     virtual bool OnRouteConnection(
+      PStringSet & routesTried,     ///< Set of routes already tried
       const PString & a_party,      ///< Source local address
       const PString & b_party,      ///< Destination indicated by source
       OpalCall & call,              ///< Call for new connection
@@ -385,6 +386,25 @@ class OpalManager : public PObject
       OpalConnection::StringOptions * stringOptions
     );
 
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OnProceeding() on the connection's
+       associated OpalCall object.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
+    );
+
     /**Call back for remote party being alerted on outgoing call.
        This function is called after the connection is informed that the
        remote endpoint is "ringing". This function is generally called
@@ -1210,12 +1230,36 @@ class OpalManager : public PObject
       OpalConnection & connection   ///< New connection just created
     );
 
-    OpalRecordManager & GetRecordManager()
-    { return recordManager; }
+    OpalRecordManager & GetRecordManager() const
+    { return *m_recordManager; }
 
-    virtual PBoolean StartRecording(const PString & callToken, const PFilePath & fn);
-    virtual bool IsRecording(const PString & callToken);
-    virtual void StopRecording(const PString & callToken);
+    /**Start recording a call.
+       Current version saves to a WAV file. It may either mix the receive and
+       transmit audio stream to a single mono file, or the streams are placed
+       into the left and right channels of a stereo WAV file.
+
+       Returns true if the call exists and there is no recording in progress
+               for the call.
+      */
+    virtual PBoolean StartRecording(
+      const PString & callToken,  ///< Call token for call to record
+      const PFilePath & filename, ///< File into which to record
+      bool mono = false           ///< Record as mono/stereo
+    );
+
+    /**Indicate if recording is currently active on call.
+      */
+    virtual bool IsRecording(
+      const PString & callToken   ///< Call token for call to check if recording
+    );
+
+    /** Stop a recording.
+        Returns true if the call does exists, that recording is active is
+                not indicated.
+      */
+    virtual bool StopRecording(
+      const PString & callToken   ///< Call token for call to stop recording
+    );
 
 #ifdef OPAL_ZRTP
     virtual bool GetZRTPEnabled() const;
@@ -1315,8 +1359,11 @@ class OpalManager : public PObject
         OpalManager & manager;
     } activeCalls;
 
-    PBoolean	 clearingAllCalls;
-    PSyncPoint   allCallsCleared;
+    PAtomicInteger m_clearingAllCallsCount;
+    PMutex         m_clearingAllCallsMutex;
+    PSyncPoint     m_allCallsCleared;
+    void InternalClearAllCalls(OpalConnection::CallEndReason reason, bool wait, bool first);
+
     PThread    * garbageCollector;
     PSyncPoint   garbageCollectExit;
     PDECLARE_NOTIFIER(PThread, OpalManager, GarbageMain);
@@ -1325,7 +1372,7 @@ class OpalManager : public PObject
     bool zrtpEnabled;
 #endif
 
-    OpalRecordManager recordManager;
+    OpalRecordManager * m_recordManager;
 
     friend OpalCall::OpalCall(OpalManager & mgr);
     friend void OpalCall::OnReleased(OpalConnection & connection);
