@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21293 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
  */
 
-#ifndef __OPAL_H323T120_H
-#define __OPAL_H323T120_H
+#ifndef OPAL_T120_H323T120_H
+#define OPAL_T120_H323T120_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -243,7 +243,7 @@ class H323_T120Channel : public H323DataChannel
 };
 
 
-#endif // __OPAL_H323T120_H
+#endif // OPAL_T120_H323T120_H
 
 
 /////////////////////////////////////////////////////////////////////////////
