@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21936 $
+ * $Revision: 22596 $
  * $Author: rjongbloed $
- * $Date: 2009-01-17 08:02:24 +0000 (Sat, 17 Jan 2009) $
+ * $Date: 2009-05-13 02:27:18 +0000 (Wed, 13 May 2009) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -135,6 +135,8 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
   public:
     MyPCSSEndPoint(MyManager & manager);
 
+    virtual OpalMediaFormatList GetMediaFormats() const;
+
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
@@ -387,7 +389,7 @@ struct StatisticsField
   double CalculateBandwidth(DWORD bytes);
   double CalculateFrameRate(DWORD frames);
   virtual StatisticsField * Clone() const = 0;
-  virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream);
+  virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics);
   virtual void GetValue(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics, wxString & value) = 0;
 
   const wxChar    * m_name;
@@ -464,8 +466,6 @@ class InCallPanel : public CallPanelBase
     void MicrophoneVolume(wxScrollEvent & event);
     void SetVolume(bool microphone, int value, bool muted);
 
-    bool GetConnections(bool user);
-
     wxButton  * m_Hold;
     wxButton  * m_SpeakerHandset;
     wxCheckBox* m_SpeakerMute;
