@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22599 $
+ * $Revision: 23037 $
  * $Author: rjongbloed $
- * $Date: 2009-05-13 04:47:49 +0000 (Wed, 13 May 2009) $
+ * $Date: 2009-07-03 05:19:05 +0000 (Fri, 03 Jul 2009) $
  */
 
 #ifdef __GNUC__
@@ -866,14 +866,14 @@ PBoolean OpalPluginFramedAudioTranscoder::ConvertSilentFrame(BYTE * buffer)
 //
 
 OpalPluginStreamedAudioTranscoder::OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codecDefn,
-                                                                     bool isEncoder,
-                                                                     unsigned inputBits,
-                                                                     unsigned outputBits)
+                                                                     bool isEncoder)
   : OpalStreamedTranscoder(GetRawPCM(codecDefn->sourceFormat, codecDefn->sampleRate),
                            GetRawPCM(codecDefn->destFormat,   codecDefn->sampleRate),
-                           inputBits, outputBits)
+                           16, 16)
   , OpalPluginTranscoder(codecDefn, isEncoder)
-{ 
+{
+  (isEncoder ? outputBitsPerSample : inputBitsPerSample) =
+                  (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos;
   comfortNoise       = (codecDef->flags & PluginCodec_ComfortNoiseMask) == PluginCodec_ComfortNoise;
   acceptEmptyPayload = (codecDef->flags & PluginCodec_ComfortNoiseMask) == PluginCodec_EmptyPayload;
   acceptOtherPayloads = (codecDef->flags & PluginCodec_OtherPayloadMask) == PluginCodec_OtherPayload;
@@ -884,39 +884,13 @@ PBoolean OpalPluginStreamedAudioTranscoder::UpdateMediaFormats(const OpalMediaFo
   return OpalStreamedTranscoder::UpdateMediaFormats(input, output) && UpdateOptions(isEncoder ? output : input);
 }
 
-
-OpalPluginStreamedAudioEncoder::OpalPluginStreamedAudioEncoder(const PluginCodec_Definition * codecDefn, bool)
-  : OpalPluginStreamedAudioTranscoder(codecDefn, true,
-                                      16,
-                                      (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos)
+int OpalPluginStreamedAudioTranscoder::ConvertOne(int from) const
 {
-}
-
-int OpalPluginStreamedAudioEncoder::ConvertOne(int _sample) const
-{
-  short sample = (short)_sample;
-  unsigned int fromLen = sizeof(sample);
+  unsigned int fromLen = sizeof(from);
   int to;
   unsigned toLen = sizeof(to);
   unsigned flags = 0;
-  return Transcode(&sample, &fromLen, &to, &toLen, &flags) ? to : -1;
-}
-
-
-OpalPluginStreamedAudioDecoder::OpalPluginStreamedAudioDecoder(const PluginCodec_Definition * codecDefn, bool)
-  : OpalPluginStreamedAudioTranscoder(codecDefn, false,
-                                      (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos,
-                                      16)
-{
-}
-
-int OpalPluginStreamedAudioDecoder::ConvertOne(int codedSample) const
-{
-  unsigned int fromLen = sizeof(codedSample);
-  short to;
-  unsigned toLen = sizeof(to);
-  unsigned flags = 0;
-  return Transcode(&codedSample, &fromLen, &to, &toLen, &flags) ? to : -1;
+  return Transcode(&from, &fromLen, &to, &toLen, &flags) ? to : -1;
 }
 
 
@@ -928,6 +902,7 @@ OpalPluginVideoTranscoder::OpalPluginVideoTranscoder(const PluginCodec_Definitio
   : OpalVideoTranscoder(codecDefn->sourceFormat, codecDefn->destFormat)
   , OpalPluginTranscoder(codecDefn, isEncoder)
   , m_bufferRTP(NULL)
+  , m_lastVideoFastUpdate(PTimer::Tick())
 #if PTRACING
   , m_consecutiveIntraFrames(0)
 #endif
@@ -1002,12 +977,14 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
     if (!lastFrameWasIFrame)
       m_consecutiveIntraFrames = 0;
     else if (forceIFrame)
-      PTRACE(3, "OpalPlugin\tEncoder sent forced I-Frame");
-    else if (m_consecutiveIntraFrames < 10) {
+      PTRACE(3, "OpalPlugin\tEncoder sent forced I-Frame at " << m_totalFrames);
+    else {
+      if (m_consecutiveIntraFrames == 0) 
+        PTRACE(4, "OpalPlugin\tEncoder sending I-Frame at " << m_totalFrames);
+      else
+        PTRACE(4, "OpalPlugin\tEncoder sending " << ((m_consecutiveIntraFrames == 0) ? "" : "consecutive ") << "I-Frame at " << m_totalFrames);
       if (++m_consecutiveIntraFrames >= 10)
         PTRACE(3, "OpalPlugin\tEncoder has sent too many consecutive I-Frames - assume codec cannot do P-Frames");
-      else
-        PTRACE(4, "OpalPlugin\tEncoder resending forced I-Frame");
     }
 #endif
 
@@ -1606,7 +1583,7 @@ void OpalPluginCodecManager::RegisterCodecPlugins(unsigned int count, const Plug
         new OpalPluginTranscoderFactory<OpalPluginFramedAudioTranscoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
         break;
       case PluginCodec_MediaTypeAudioStreamed:
-        new OpalPluginTranscoderFactory<OpalPluginStreamedAudioEncoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
+        new OpalPluginTranscoderFactory<OpalPluginStreamedAudioTranscoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
         break;
   #if OPAL_T38_CAPABILITY
       case PluginCodec_MediaTypeFax:
