@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22011 $
+ * $Revision: 22220 $
  * $Author: rjongbloed $
- * $Date: 2009-02-06 08:17:03 +0000 (Fri, 06 Feb 2009) $
+ * $Date: 2009-03-18 22:51:48 +0000 (Wed, 18 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -156,6 +156,12 @@ PBoolean OpalRFC2833Proto::SendToneAsync(char tone, unsigned duration)
     }
   }
 
+  // if transmittter is ever in this state, then stop the duration timer
+  if (m_payloadType == RTP_DataFrame::IllegalPayloadType) {
+    PTRACE(2, "RFC2833\tNo payload type, cannot send packet.");
+    return false;
+  }
+
   // convert tone to correct code
   PINDEX code = ASCIIToRFC2833(tone, m_txCapabilitySet[NSECodeBase]);
 
