@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21969 $
+ * $Revision: 22596 $
  * $Author: rjongbloed $
- * $Date: 2009-01-30 23:05:54 +0000 (Fri, 30 Jan 2009) $
+ * $Date: 2009-05-13 02:27:18 +0000 (Wed, 13 May 2009) $
  */
 
 #include <ptlib.h>
@@ -46,6 +46,7 @@
 #include <h323/h323con.h>
 #include <h323/h323pdu.h>
 #include <h323/transaddr.h>
+#include <t38/h323t38.h>
 
 
 #define DEFINE_G711_CAPABILITY(cls, code, capName) \
@@ -55,11 +56,15 @@ class cls : public H323_G711Capability { \
 }; \
 H323_REGISTER_CAPABILITY(cls, capName) \
 
-#ifndef NO_H323_AUDIO_CODECS
 
+#ifndef NO_H323_AUDIO_CODECS
 DEFINE_G711_CAPABILITY(H323_G711ALaw64Capability, H323_G711Capability::ALaw, OPAL_G711_ALAW_64K)
 DEFINE_G711_CAPABILITY(H323_G711uLaw64Capability, H323_G711Capability::muLaw, OPAL_G711_ULAW_64K)
+#endif
 
+
+#if OPAL_T38_CAPABILITY
+H323_REGISTER_CAPABILITY(H323_T38Capability, OPAL_T38);
 #endif
 
 
@@ -164,6 +169,7 @@ PBoolean H323Capability::IsMatch(const PASN_Choice & subTypePDU) const
 
 PBoolean H323Capability::OnSendingPDU(H245_DataType & /*pdu*/) const
 {
+  mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
   return mediaFormat.ToCustomisedOptions();
 }
 
@@ -200,12 +206,14 @@ PBoolean H323Capability::OnReceivedPDU(const H245_Capability & cap)
       capabilityDirection = e_NoDirection;
   }
 
-  return PTrue;
+  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
+  return mediaFormat.ToNormalisedOptions();
 }
 
 
 PBoolean H323Capability::OnReceivedPDU(const H245_DataType & /*pdu*/, PBoolean /*receiver*/)
 {
+  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
   return mediaFormat.ToNormalisedOptions();
 }
 
@@ -1775,8 +1783,7 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
               }
             }
           }
-          if (copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability) &&
-              copy->GetWritableMediaFormat().ToNormalisedOptions())
+          if (copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability))
             table.Append(copy);
           else
             delete copy;
