@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 21806 $
+ * $Revision: 22361 $
  * $Author: rjongbloed $
- * $Date: 2008-12-16 02:52:00 +0000 (Tue, 16 Dec 2008) $
+ * $Date: 2009-04-01 07:06:50 +0000 (Wed, 01 Apr 2009) $
  */
 
 #include <ptlib.h>
@@ -279,7 +279,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
   if (gatekeeper != NULL) {
     PBoolean same = PTrue;
 
-    if (!address)
+    if (!address && address != "*")
       same = gatekeeper->GetTransport().GetRemoteAddress().IsEquivalent(address);
 
     if (!same && !identifier)
@@ -303,7 +303,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
       transport = new H323TransportUDP(*this, ip, port);
   }
 
-  if (address.IsEmpty()) {
+  if (address.IsEmpty() || address == "*") {
     if (identifier.IsEmpty())
       return DiscoverGatekeeper(transport);
     else
@@ -605,10 +605,10 @@ PBoolean H323EndPoint::InternalMakeCall(OpalCall & call,
   if (gatekeeper != NULL)
     transport = gatekeeper->GetTransport().GetLocalAddress().CreateTransport(
                                           *this, OpalTransportAddress::Streamed);
-  else if (stringOptions == NULL || !stringOptions->Contains("interface"))
+  else if (stringOptions == NULL || !stringOptions->Contains(OPAL_OPT_INTERFACE))
     transport = address.CreateTransport(*this, OpalTransportAddress::NoBinding);
   else {
-    OpalTransportAddress localInterface = (*stringOptions)["interface"];
+    OpalTransportAddress localInterface = (*stringOptions)[OPAL_OPT_INTERFACE];
     transport = localInterface.CreateTransport(*this, OpalTransportAddress::HostOnly);
   }
 
