@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21952 $
+ * $Revision: 22699 $
  * $Author: rjongbloed $
- * $Date: 2009-01-27 03:02:25 +0000 (Tue, 27 Jan 2009) $
+ * $Date: 2009-05-22 05:39:55 +0000 (Fri, 22 May 2009) $
  */
 
 #include <ptlib.h>
@@ -137,28 +137,24 @@ OpalMediaFormatList OpalPCSSEndPoint::GetMediaFormats() const
   return list;
 }
 
-static PBoolean SetDeviceName(const PString & name,
+static bool SetDeviceName(const PString & name,
                           PSoundChannel::Directions dir,
                           PString & result)
 {
-  PStringArray devices = PSoundChannel::GetDeviceNames(dir);
-
-  if (name.GetLength() >= 2 && name[0] == '#' && isdigit(name[1])) {
-    PINDEX id = name.Mid(1).AsUnsigned();
-    if (id > 0 && id <= devices.GetSize()) {
-      result = devices[id-1];
-      return true;
-    }
-  }
-
-  if (devices.GetValuesIndex(name) != P_MAX_INDEX) {
-    // Perfect match (possibly including driver name)
+  // First see if the channel can be created from the name alone, this
+  // picks up *.wav style names.
+  PSoundChannel * channel = PSoundChannel::CreateChannelByName(name, dir);
+  if (channel != NULL) {
+    delete channel;
     result = name;
     return true;
   }
 
-  // Create unique names sans driver
+  // Otherwise, look for a partial match.
+
+  // Need to create unique names sans driver
   PStringList uniqueNames;
+  PStringArray devices = PSoundChannel::GetDeviceNames(dir);
   for (PINDEX i = 0; i < devices.GetSize(); ++i) {
     PCaselessString deviceName = devices[i];
     PINDEX tab = deviceName.Find('\t');
@@ -254,14 +250,6 @@ OpalPCSSConnection * OpalPCSSEndPoint::CreateConnection(OpalCall & call,
   return new OpalPCSSConnection(call, *this, playDevice, recordDevice, options, stringOptions);
 }
 
-OpalPCSSConnection * OpalPCSSEndPoint::CreateConnection(OpalCall & call,
-                                                        const PString & playDevice,
-                                                        const PString & recordDevice,
-                                                        void * userData)
-{
-  return CreateConnection(call, playDevice, recordDevice, userData, 0, NULL);
-}
-
 
 PSoundChannel * OpalPCSSEndPoint::CreateSoundChannel(const OpalPCSSConnection & connection,
 						                                            const OpalMediaFormat & mediaFormat,
