@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22619 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-15 00:20:58 +0000 (Fri, 15 May 2009) $
  */
 
 #include <ptlib.h>
@@ -48,6 +48,7 @@
 
 #define new PNEW
 
+#define BAD_TRANSMIT_TIME_MAX 10    //  maximum of seconds of transmit fails before session is killed
 
 const unsigned SecondsFrom1900to1970 = (70*365+17)*24*60*60U;
 
@@ -72,10 +73,10 @@ static PFactory<RTP_Encoding>::Worker<RTP_Encoding> rtpAVPHandler("rtp/avp");
 
 /////////////////////////////////////////////////////////////////////////////
 
-RTP_DataFrame::RTP_DataFrame(PINDEX sz)
-  : PBYTEArray(MinHeaderSize+sz)
+RTP_DataFrame::RTP_DataFrame(PINDEX payloadSz, PINDEX bufferSz)
+  : PBYTEArray(std::max(bufferSz, MinHeaderSize+payloadSz))
 {
-  payloadSize = sz;
+  payloadSize = payloadSz;
   theArray[0] = '\x80';
 }
 
@@ -451,6 +452,14 @@ OpalMediaStatistics::OpalMediaStatistics()
 {
 }
 
+#if OPAL_FAX
+OpalMediaStatistics::Fax::Fax()
+{
+  memset(this, 0, sizeof(*this));
+  m_result = -2;
+}
+#endif
+
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
@@ -464,7 +473,7 @@ void RTP_UserData::OnRxStatistics(const RTP_Session & /*session*/) const
 {
 }
 
-void RTP_UserData::OnClearCall(const RTP_Session & /*session*/)
+void RTP_UserData::SessionFailing(RTP_Session & /*session*/)
 {
 }
 
@@ -482,13 +491,11 @@ void RTP_UserData::OnTxIntraFrameRequest(const RTP_Session & /*session*/) const
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_Session::RTP_Session(const Params & params)
-: canonicalName(PProcess::Current().GetUserName()),
-  toolName(PProcess::Current().GetName()),
-  reportTimeInterval(0, 12),  // Seconds
-  reportTimer(reportTimeInterval)
-#if OPAL_RTP_AGGREGATE
-  , aggregator(params.aggregator)
-#endif
+  : canonicalName(PProcess::Current().GetUserName())
+  , toolName(PProcess::Current().GetName())
+  , reportTimeInterval(0, 12)  // Seconds
+  , reportTimer(reportTimeInterval)
+  , failed(false)
 {
   PAssert(params.id > 0 && params.id < 256, PInvalidParameter);
   sessionID = (BYTE)params.id;
@@ -496,7 +503,6 @@ RTP_Session::RTP_Session(const Params & params)
 
   userData = params.userData;
   autoDeleteUserData = params.autoDelete;
-  jitter = NULL;
 
   ignoreOutOfOrderPackets = true;
   ignorePayloadTypeChanges = true;
@@ -518,34 +524,7 @@ RTP_Session::RTP_Session(const Params & params)
   lastRRSequenceNumber = 0;
   consecutiveOutOfOrderPackets = 0;
 
-  packetsSent = 0;
-  rtcpPacketsSent = 0;
-  octetsSent = 0;
-  packetsReceived = 0;
-  octetsReceived = 0;
-  packetsLost = 0;
-  packetsOutOfOrder = 0;
-  averageSendTime = 0;
-  maximumSendTime = 0;
-  minimumSendTime = 0;
-  averageReceiveTime = 0;
-  maximumReceiveTime = 0;
-  minimumReceiveTime = 0;
-  jitterLevel = 0;
-  maximumJitterLevel = 0;
-  markerRecvCount = 0;
-  markerSendCount = 0;
-
-  txStatisticsCount = 0;
-  rxStatisticsCount = 0;
-  averageSendTimeAccum = 0;
-  maximumSendTimeAccum = 0;
-  minimumSendTimeAccum = 0xffffffff;
-  averageReceiveTimeAccum = 0;
-  maximumReceiveTimeAccum = 0;
-  minimumReceiveTimeAccum = 0xffffffff;
-  packetsLostSinceLastRR = 0;
-  lastTransitTime = 0;
+  ClearStatistics();
 
   lastReceivedPayloadType = RTP_DataFrame::IllegalPayloadType;
 
@@ -579,12 +558,45 @@ RTP_Session::~RTP_Session()
       "    averageJitter     = " << (jitterLevel >> 7) << "\n"
       "    maximumJitter     = " << (maximumJitterLevel >> 7)
      );
-  delete jitter;
   if (autoDeleteUserData)
     delete userData;
   delete m_encodingHandler;
 }
 
+
+void RTP_Session::ClearStatistics()
+{
+  packetsSent = 0;
+  rtcpPacketsSent = 0;
+  octetsSent = 0;
+  packetsReceived = 0;
+  octetsReceived = 0;
+  packetsLost = 0;
+  packetsOutOfOrder = 0;
+  averageSendTime = 0;
+  maximumSendTime = 0;
+  minimumSendTime = 0;
+  averageReceiveTime = 0;
+  maximumReceiveTime = 0;
+  minimumReceiveTime = 0;
+  jitterLevel = 0;
+  maximumJitterLevel = 0;
+  markerRecvCount = 0;
+  markerSendCount = 0;
+
+  txStatisticsCount = 0;
+  rxStatisticsCount = 0;
+  averageSendTimeAccum = 0;
+  maximumSendTimeAccum = 0;
+  minimumSendTimeAccum = 0xffffffff;
+  averageReceiveTimeAccum = 0;
+  maximumReceiveTimeAccum = 0;
+  minimumReceiveTimeAccum = 0xffffffff;
+  packetsLostSinceLastRR = 0;
+  lastTransitTime = 0;
+}
+
+
 void RTP_Session::SendBYE()
 {
   {
@@ -682,41 +694,38 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
                                         PINDEX stackSize)
 {
   if (minJitterDelay == 0 && maxJitterDelay == 0) {
-    PTRACE(4, "InfLID\tSwitching off jitter buffer.");
-    if (jitter != NULL) {
-      delete jitter;
-      jitter = NULL;
-    }
+    PTRACE_IF(4, m_jitterBuffer != NULL, "InfLID\tSwitching off jitter buffer.");
+    m_jitterBuffer.SetNULL();
   }
   else {
     PTRACE(4, "InfLID\tSetting jitter buffer time from " << minJitterDelay << " to " << maxJitterDelay);
     SetIgnoreOutOfOrderPackets(false);
-    if (jitter != NULL) 
-      jitter->SetDelay(minJitterDelay, maxJitterDelay);
+    if (m_jitterBuffer != NULL)
+      m_jitterBuffer->SetDelay(minJitterDelay, maxJitterDelay);
     else
-      jitter = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, timeUnits, stackSize);
-    jitter->Resume(
-#if OPAL_RTP_AGGREGATE
-      aggregator
-#endif
-    );
+      m_jitterBuffer = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, timeUnits, stackSize);
+    m_jitterBuffer->Resume();
   }
 }
 
 
 unsigned RTP_Session::GetJitterBufferSize() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetJitterTime() : 0;
 }
 
+
 unsigned RTP_Session::GetJitterTimeUnits() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetTimeUnits() : 0;
 }
 
 
 PBoolean RTP_Session::ReadBufferedData(RTP_DataFrame & frame)
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->ReadData(frame) : ReadData(frame, true);
 }
 
@@ -1002,6 +1011,8 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
     else if (allowSequenceChange) {
       expectedSequenceNumber = (WORD) (sequenceNumber + 1);
       allowSequenceChange = false;
+      PTRACE(2, "RTP\tSession " << sessionID << ", adjusting sequence numbers to expect "
+             << expectedSequenceNumber << " ssrc=" << syncSourceIn);
     }
     else if (sequenceNumber < expectedSequenceNumber) {
       PTRACE(2, "RTP\tSession " << sessionID << ", out of order packet, received "
@@ -1042,33 +1053,36 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
   if (!SendReport())
     return e_AbortTransport;
 
-  if (rxStatisticsCount < rxStatisticsInterval)
-    return e_ProcessPacket;
+  if (rxStatisticsCount >= rxStatisticsInterval) {
 
-  rxStatisticsCount = 0;
+    rxStatisticsCount = 0;
 
-  averageReceiveTime = averageReceiveTimeAccum/rxStatisticsInterval;
-  maximumReceiveTime = maximumReceiveTimeAccum;
-  minimumReceiveTime = minimumReceiveTimeAccum;
+    averageReceiveTime = averageReceiveTimeAccum/rxStatisticsInterval;
+    maximumReceiveTime = maximumReceiveTimeAccum;
+    minimumReceiveTime = minimumReceiveTimeAccum;
 
-  averageReceiveTimeAccum = 0;
-  maximumReceiveTimeAccum = 0;
-  minimumReceiveTimeAccum = 0xffffffff;
+    averageReceiveTimeAccum = 0;
+    maximumReceiveTimeAccum = 0;
+    minimumReceiveTimeAccum = 0xffffffff;
 
-  PTRACE(4, "RTP\tSession " << sessionID << ", receive statistics:"
-            " packets=" << packetsReceived <<
-            " octets=" << octetsReceived <<
-            " lost=" << packetsLost <<
-            " tooLate=" << GetPacketsTooLate() <<
-            " order=" << packetsOutOfOrder <<
-            " avgTime=" << averageReceiveTime <<
-            " maxTime=" << maximumReceiveTime <<
-            " minTime=" << minimumReceiveTime <<
-            " jitter=" << (jitterLevel >> 7) <<
-            " maxJitter=" << (maximumJitterLevel >> 7));
+    PTRACE(4, "RTP\tSession " << sessionID << ", receive statistics:"
+              " packets=" << packetsReceived <<
+              " octets=" << octetsReceived <<
+              " lost=" << packetsLost <<
+              " tooLate=" << GetPacketsTooLate() <<
+              " order=" << packetsOutOfOrder <<
+              " avgTime=" << averageReceiveTime <<
+              " maxTime=" << maximumReceiveTime <<
+              " minTime=" << minimumReceiveTime <<
+              " jitter=" << (jitterLevel >> 7) <<
+              " maxJitter=" << (maximumJitterLevel >> 7));
 
-  if (userData != NULL)
-    userData->OnRxStatistics(*this);
+    if (userData != NULL)
+      userData->OnRxStatistics(*this);
+  }
+
+  for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) 
+    f->notifier(frame, (INT)this);
 
   return e_ProcessPacket;
 }
@@ -1449,12 +1463,14 @@ void RTP_Session::SourceDescription::PrintOn(ostream & strm) const
 
 DWORD RTP_Session::GetPacketsTooLate() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetPacketsTooLate() : 0;
 }
 
 
 DWORD RTP_Session::GetPacketOverruns() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetBufferOverruns() : 0;
 }
 
@@ -1464,6 +1480,17 @@ PBoolean RTP_Session::WriteOOBData(RTP_DataFrame &, bool)
   return true;
 }
 
+void RTP_Session::AddFilter(const PNotifier & filter)
+{
+  // ensures that a filter is added only once
+  for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
+    if (f->notifier == filter)
+      return;
+  }
+  filters.Append(new Filter(filter));
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 static void SetMinBufferSize(PUDPSocket & sock, int buftype)
@@ -1512,8 +1539,7 @@ RTP_UDP::~RTP_UDP()
   // We need to do this to make sure that the sockets are not
   // deleted before select decides there is no more data coming
   // over them and exits the reading thread.
-  if (jitter)
-    PAssert(jitter->WaitForTermination(20000), "Jitter buffer thread did not terminate");
+  SetJitterBufferSize(0, 0);
 
   delete dataSocket;
   delete controlSocket;
@@ -1774,7 +1800,6 @@ PBoolean RTP_UDP::Internal_ReadData(RTP_DataFrame & frame, PBoolean loop)
       PWaitAndSignal mutex(dataMutex);
       if (shutdownRead) {
         PTRACE(3, "RTP_UDP\tSession " << sessionID << ", Read shutdown.");
-        shutdownRead = false;
         return false;
       }
     }
@@ -1914,15 +1939,16 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
   switch (socket.GetErrorNumber()) {
     case ECONNRESET :
     case ECONNREFUSED :
-      if (++badTransmitCounter < 25) { 
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
-        return RTP_Session::e_IgnorePacket;
-      }
+      PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
+      if (++badTransmitCounter == 1) 
+        badTransmitStart = PTime();
       else {
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " too many transmit fails - killing call");
-        userData->OnClearCall(*this);
-        return RTP_Session::e_AbortTransport;
+        if ((PTime()- badTransmitStart).GetMilliSeconds() < BAD_TRANSMIT_TIME_MAX * 1000)
+          return RTP_Session::e_IgnorePacket;
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " " << BAD_TRANSMIT_TIME_MAX << "s of transmit fails - informing connection");
+        userData->SessionFailing(*this);
       }
+      return RTP_Session::e_IgnorePacket;
 
     case EMSGSIZE :
       PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName
@@ -2037,7 +2063,6 @@ PBoolean RTP_UDP::Internal_WriteData(RTP_DataFrame & frame)
     PWaitAndSignal mutex(dataMutex);
     if (shutdownWrite) {
       PTRACE(3, "RTP_UDP\tSession " << sessionID << ", write shutdown.");
-      shutdownWrite = false;
       return false;
     }
   }
@@ -2147,6 +2172,8 @@ void RTP_Session::SetEncoding(const PString & newEncoding)
     m_encodingHandler = newHandler;
   }
 
+  ClearStatistics();
+
   EncodingLock(*this)->OnStart(*this);
 }
 
