@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 22178 $
+ * $Revision: 23028 $
  * $Author: rjongbloed $
- * $Date: 2009-03-09 10:04:21 +0000 (Mon, 09 Mar 2009) $
+ * $Date: 2009-07-02 10:39:41 +0000 (Thu, 02 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -174,9 +174,13 @@ class T38PseudoRTP_Handler : public RTP_Encoding
         if (!udptl.Decode(rawData)) {
     #if PTRACING
           if (oneGoodPacket)
-            PTRACE(2, "RTP_T38\tRaw data decode failure:\n  "
-                   << setprecision(2) << rawData << "\n  UDPTL = "
-                   << setprecision(2) << udptl);
+            if (PTrace::CanTrace(5))
+              PTRACE(5, "RTP_T38\tRaw data decode failure:\n  "
+                     << setprecision(2) << rawData << "\n  UDPTL = "
+                     << setprecision(2) << udptl);
+            else
+              PTRACE(2, "RTP_T38\tRaw data decode failure:\n  "
+                     << setprecision(2) << rawData);
           else
             PTRACE(2, "RTP_T38\tRaw data decode failure: " << rawData.GetSize() << " bytes.");
     #endif
@@ -223,9 +227,10 @@ typedef std::map<std::string, OpalFaxCallInfo *> OpalFaxCallInfoMap_T;
 static OpalFaxCallInfoMap_T faxCallInfoMap;
 
 OpalFaxCallInfo::OpalFaxCallInfo()
-{ 
-  refCount = 1; 
-  spanDSPPort = 0; 
+  : stdoutThread(NULL)
+  , refCount(1)
+  , spanDSPPort(0)
+{
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -246,13 +251,13 @@ OpalFaxMediaStream::OpalFaxMediaStream(OpalFaxConnection & conn,
   , m_receive(receive)
   , m_stationId(stationId)
 {
-  SetDataSize(RTP_DataFrame::MaxMtuPayloadSize);
+  SetDataSize(RTP_DataFrame::MaxMtuPayloadSize, 1);
 }
 
 PBoolean OpalFaxMediaStream::Open()
 {
   if (sessionToken.IsEmpty()) {
-    PTRACE(1, "T38\tCannot open unknown media stream");
+    PTRACE(1, "Fax\tCannot open unknown media stream");
     return false;
   }
 
@@ -292,38 +297,30 @@ PBoolean OpalFaxMediaStream::Open()
 #if _WIN32
     cmdLine.Replace("\\", "\\\\", true);
 #endif
-    
+
     PTRACE(1, "Fax\tExecuting '" << cmdLine << "'");
 
     // open connection to spandsp
-    if (!m_faxCallInfo->spanDSP.Open(cmdLine, PPipeChannel::ReadOnly, false, true)) {
+    if (!m_faxCallInfo->spanDSP.Open(cmdLine, PPipeChannel::ReadOnly, false, false)) {
       PTRACE(1, "Fax\tCannot open SpanDSP: " << m_faxCallInfo->spanDSP.GetErrorText());
       return false;
     }
 
-#if PTRACING
-    PString errmsg;
-    while (m_faxCallInfo->spanDSP.ReadStandardError(errmsg, false))
-      PTRACE(1, "Fax\tspandsp_util: " << errmsg);
-#endif
-
     if (!m_faxCallInfo->spanDSP.Execute()) {
       PTRACE(1, "Fax\tCannot execute SpanDSP: return code=" << m_faxCallInfo->spanDSP.GetReturnCode());
       return false;
     }
   }
 
+  if (IsSink() && m_faxCallInfo->stdoutThread == NULL)
+    m_faxCallInfo->stdoutThread = PThread::Create(PCREATE_NOTIFIER(ReadStdOut), "SpanDSP Output");
+
   return OpalMediaStream::Open();
 }
 
+
 PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
-#if PTRACING
-    PString errmsg;
-    while (m_faxCallInfo->spanDSP.ReadStandardError(errmsg, false))
-      PTRACE(1, "Fax\tspandsp_util: " << errmsg);
-#endif
-
   // it is possible for ReadPacket to be called before the media stream has been opened, so deal with that case
   PWaitAndSignal m(infoMutex);
   if ((m_faxCallInfo == NULL) || !m_faxCallInfo->spanDSP.IsRunning()) {
@@ -351,9 +348,6 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
     packet.SetPayloadType(RTP_DataFrame::MaxPayloadType);
     packet.SetPayloadSize(len);
 
-    if (len > 0)
-      m_connection.CheckFaxStopped();
-
 #if WRITE_PCM_FILE
     static int file = _open("t38_audio_in.pcm", _O_BINARY | _O_CREAT | _O_TRUNC | _O_WRONLY, _S_IREAD | _S_IWRITE);
     if (file >= 0) {
@@ -435,8 +429,6 @@ PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
         }
         writeBufferLen = 0;
       }
-
-      m_connection.CheckFaxStopped();
     }
   }
 
@@ -448,24 +440,42 @@ PBoolean OpalFaxMediaStream::Close()
   if (!OpalMediaStream::Close())
     return false;
 
+  // Give the spandsp sub-process a second to tidy up and exit with
+  // some statistics.
+  if (m_faxCallInfo->stdoutThread != NULL) {
+    PTRACE(4, "Fax\tAwaiting final statistics from SpanDSP");
+    if (!m_faxCallInfo->stdoutThread->WaitForTermination(2000)) {
+      // OK, force the issue
+      m_faxCallInfo->spanDSP.Close();
+      m_faxCallInfo->stdoutThread->WaitForTermination(1000);
+    }
+  }
+
   OpalFaxCallInfo * faxCallInfo;
   {
-    PWaitAndSignal m1(infoMutex);
+    {
+      PWaitAndSignal m1(infoMutex);
 
-    if (m_faxCallInfo == NULL || sessionToken.IsEmpty()) {
-      PTRACE(1, "Fax\tCannot close unknown media stream");
-      return true;
+      if (m_faxCallInfo == NULL || sessionToken.IsEmpty()) {
+        PTRACE(1, "Fax\tCannot close unknown media stream");
+        return true;
+      }
+
+      faxCallInfo = m_faxCallInfo;
+      m_faxCallInfo = NULL;
     }
 
-    faxCallInfo = m_faxCallInfo;
-    m_faxCallInfo = NULL;
-
-    PWaitAndSignal m2(faxMapMutex);
-
     // shutdown whatever is running
     faxCallInfo->socket.Close();
     faxCallInfo->spanDSP.Close();
 
+    PWaitAndSignal m2(faxMapMutex);
+
+    if (faxCallInfo->stdoutThread != NULL) {
+      delete faxCallInfo->stdoutThread;
+      faxCallInfo->stdoutThread = NULL;
+    }
+
     OpalFaxCallInfoMap_T::iterator r = faxCallInfoMap.find(sessionToken);
     if (r == faxCallInfoMap.end()) {
       PTRACE(1, "Fax\tError: media stream not found in T38 session list");
@@ -504,6 +514,92 @@ PBoolean OpalFaxMediaStream::IsSynchronous() const
   return IsSource();
 }
 
+
+void OpalFaxMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
+{
+  OpalMediaStream::GetStatistics(statistics, fromPatch);
+  statistics.m_fax = m_statistics;
+}
+
+
+static bool ExtractValue(const PString & msg, PINDEX & position, int & value, char sep = '=')
+{
+  position = msg.Find(sep, position+1);
+  if (position == P_MAX_INDEX)
+    return false;
+
+  PINDEX eol = msg.Find('\n', position);
+  if (eol == P_MAX_INDEX)
+    return false;
+
+  value = msg(position+1, eol-1).AsInteger();
+  return true;
+}
+
+
+void OpalFaxMediaStream::ReadStdOut(PThread &, INT)
+{
+  PTRACE(4, "Fax\tSpanDSP stdout reading thread started");
+
+  PString msg;
+  bool notInStats = true;
+
+  for (;;) {
+    int c = m_faxCallInfo->spanDSP.ReadChar();
+    if (c < 0) {
+      PTRACE(2, "Fax\tError reading from " << m_faxCallInfo->spanDSP.GetName()
+             << ": " << m_faxCallInfo->spanDSP.GetErrorText(PChannel::LastReadError));
+      m_connection.OnFaxCompleted(m_statistics.m_result != 0);
+      return;
+    }
+
+    if (c != '\r' && c != '\n') {
+      msg += (char)c;
+      continue;
+    }
+
+    if (notInStats) {
+      notInStats = msg.Find("statistics") == P_MAX_INDEX;
+      if (notInStats) {
+        PTRACE_IF(4, !msg.IsEmpty(), "Fax\tSpanDSP Output: \"" << msg << '"');
+        msg.MakeEmpty();
+        continue;
+      }
+    }
+
+    if (msg.Find("----------") == P_MAX_INDEX) {
+      msg += (char)c;
+      continue;
+    }
+
+    int result, errorCorrection;
+    PINDEX position = 0;
+    if (ExtractValue(msg, position, result) &&
+        ExtractValue(msg, position, m_statistics.m_bitRate) &&
+        ExtractValue(msg, position, m_statistics.m_compression) &&
+        ExtractValue(msg, position, errorCorrection) &&
+        ExtractValue(msg, position, m_statistics.m_txPages) &&
+        ExtractValue(msg, position, m_statistics.m_rxPages) &&
+        ExtractValue(msg, position, m_statistics.m_totalPages) &&
+        ExtractValue(msg, position, m_statistics.m_imageSize) &&
+        ExtractValue(msg, position, m_statistics.m_resolutionX) &&
+        ExtractValue(msg, position, m_statistics.m_resolutionY, 'x') &&
+        ExtractValue(msg, position, m_statistics.m_pageWidth) &&
+        ExtractValue(msg, position, m_statistics.m_pageHeight, 'x') &&
+        ExtractValue(msg, position, m_statistics.m_badRows) &&
+        ExtractValue(msg, position, m_statistics.m_mostBadRows) &&
+        ExtractValue(msg, position, m_statistics.m_errorCorrectionRetries))
+    {
+      m_statistics.m_result = result; // Only set this if everything parsed correctly
+      m_statistics.m_errorCorrection = errorCorrection != 0;
+    }
+    PTRACE(4, "Fax\tSpanDSP Output:\n" << msg);
+    notInStats = true;
+    msg.MakeEmpty();
+  }
+}
+
+
 PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
 {
   PStringStream cmdline;
@@ -514,8 +610,15 @@ PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
   cmdline << ((OpalFaxEndPoint &)connection.GetEndPoint()).GetSpanDSP() << " -m ";
   if (m_receive)
     cmdline << "fax_to_tiff";
-  else
+  else {
     cmdline << "tiff_to_fax";
+    if (!m_stationId.IsEmpty())
+      cmdline << " -s '" << m_stationId << "'";
+  }
+#if PTRACING
+  if (PTrace::CanTrace(5))
+    cmdline << " -v";
+#endif
   cmdline << " -V 0 -n '" << m_filename << "' -f 127.0.0.1:" << port;
 
   return cmdline;
@@ -545,7 +648,7 @@ PString OpalT38MediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
   PIPSocket::Address dummy; WORD port;
   info.socket.GetLocalAddress(dummy, port);
 
-  cmdline << ((OpalFaxEndPoint &)connection.GetEndPoint()).GetSpanDSP() << " -V 0 -m ";
+  cmdline << ((OpalFaxEndPoint &)connection.GetEndPoint()).GetSpanDSP() << " -m ";
   if (m_receive)
     cmdline << "t38_to_tiff";
   else {
@@ -553,7 +656,11 @@ PString OpalT38MediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
     if (!m_stationId.IsEmpty())
       cmdline << " -s '" << m_stationId << "'";
   }
-  cmdline << " -v -n '" << m_filename << "' -t 127.0.0.1:" << port;
+#if PTRACING
+  if (PTrace::CanTrace(5))
+    cmdline << " -v";
+#endif
+  cmdline << " -V 0 -n '" << m_filename << "' -t 127.0.0.1:" << port;
 
   return cmdline;
 }
@@ -579,15 +686,14 @@ PBoolean OpalT38MediaStream::ReadPacket(RTP_DataFrame & packet)
       stat = m_faxCallInfo->socket.Read(packet.GetPointer(), packet.GetSize());
     else {
       stat = m_faxCallInfo->socket.ReadFrom(packet.GetPointer(), packet.GetSize(), m_faxCallInfo->spanDSPAddr, m_faxCallInfo->spanDSPPort);
-      PTRACE(2, "Fax\tRemote spandsp address set to " << m_faxCallInfo->spanDSPAddr << ":" << m_faxCallInfo->spanDSPPort);
+      PTRACE_IF(2, stat, "Fax\tRemote spandsp address set to " << m_faxCallInfo->spanDSPAddr << ":" << m_faxCallInfo->spanDSPPort);
     }
 
     if (!stat) {
-      if (m_faxCallInfo->socket.GetErrorCode(PChannel::LastReadError) == PChannel::Timeout) {
-        packet.SetPayloadSize(0);
-        return true;
-      }
-      return false;
+      if (m_faxCallInfo->socket.GetErrorCode(PChannel::LastReadError) != PChannel::Timeout)
+        return false;
+      packet.SetPayloadSize(0);
+      return true;
     }
 
     PINDEX len = m_faxCallInfo->socket.GetLastReadCount();
@@ -596,7 +702,6 @@ PBoolean OpalT38MediaStream::ReadPacket(RTP_DataFrame & packet)
 
     packet.SetSize(len);
     packet.SetPayloadSize(len - RTP_DataFrame::MinHeaderSize);
-    m_connection.CheckFaxStopped();
   }
 
   return true;
@@ -627,7 +732,6 @@ PBoolean OpalT38MediaStream::WritePacket(RTP_DataFrame & packet)
       PTRACE(2, "T38_UDP\tSocket write error - " << m_faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
       return false;
     }
-    m_connection.CheckFaxStopped();
   }
 
   return true;
@@ -680,6 +784,7 @@ OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
   formats += OpalPCM16;
+  formats += OpalT38;
   return formats;
 }
 
@@ -692,6 +797,13 @@ void OpalFaxEndPoint::AcceptIncomingConnection(const PString & token)
 }
 
 
+void OpalFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool failed)
+{
+  PTRACE(3, "FAX\tFax " << (failed ? "failed" : "completed") << " on connection: " << connection);
+  connection.Release(failed ? OpalConnection::EndedByCapabilityExchange : OpalConnection::EndedByLocalUser);
+}
+
+
 PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
                                 const PString & remoteParty,
                                          void * userData,
@@ -731,7 +843,7 @@ PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
   if ((*stringOptions)("stationid").IsEmpty())
     stringOptions->SetAt("stationid", stationId);
 
-  stringOptions->SetAt("Disable-Jitter", "1");
+  stringOptions->SetAt(OPAL_OPT_DISABLE_JITTER, "1");
 
   return AddConnection(CreateConnection(call, userData, stringOptions, filename, receive,
                                         remoteParty.Left(prefixLength) *= m_t38Prefix));
@@ -751,9 +863,8 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
   , m_filename(filename)
   , m_receive(receive)
 {
+  synchronousOnRelease = false;
   PTRACE(3, "FAX\tCreated FAX connection with token \"" << callToken << '"');
-
-  m_faxStopped.SetNotifier(PCREATE_NOTIFIER(OnFaxStoppedTimeout));
 }
 
 
@@ -847,16 +958,9 @@ void OpalFaxConnection::AcceptIncoming()
 }
 
 
-void OpalFaxConnection::CheckFaxStopped()
+void OpalFaxConnection::OnFaxCompleted(bool failed)
 {
-  m_faxStopped.SetInterval(0, 30);
-}
-
-
-void OpalFaxConnection::OnFaxStoppedTimeout(PTimer &, INT)
-{
-  PTRACE(3, "FAX\tTimeout on fax connection: " << GetToken());
-  Release();
+  m_endpoint.OnFaxCompleted(*this, failed);
 }
 
 
@@ -933,15 +1037,16 @@ OpalMediaStream * OpalT38Connection::CreateMediaStream(const OpalMediaFormat & m
 }
 
 
-void OpalT38Connection::OnClosedMediaStream(const OpalMediaStream & stream)
+void OpalT38Connection::OnMediaPatchStop(unsigned sessionId, bool isSource)
 {
-  m_faxMode = stream.GetMediaFormat().GetMediaType() != OpalMediaType::Fax();
+  OpalMediaStreamPtr stream = GetMediaStream(sessionId, isSource);
+  bool newMode = stream->GetMediaFormat().GetMediaType() != OpalMediaType::Fax();
+  if (m_faxMode != newMode) {
+    m_faxTimer.Stop();
+    m_faxMode = newMode;
+  }
 
-  if (m_syncMode == Mode_UserInput)
-    OnUserInputTone(' ', 0);
-  m_faxTimer.Stop();
-
-  OpalFaxConnection::OnClosedMediaStream(stream);
+  OpalConnection::OnMediaPatchStop(sessionId, isSource);
 }
 
 
@@ -966,16 +1071,18 @@ PBoolean OpalT38Connection::SendUserInputTone(char tone, unsigned /*duration*/)
 
 void OpalT38Connection::OnSendCNGCED(PTimer & timer, INT)
 {
-  if (LockReadOnly() && !m_faxMode && m_syncMode == Mode_UserInput) {
-    if (m_receive) {
-      // Cadence for CED is single tone, but we repeat just in case
-      OnUserInputTone('Y', 3600);
-      timer = 5000;
-    }
-    else {
-      // Cadence for CNG is 500ms on 3 seconds off
-      OnUserInputTone('X', 500);
-      timer = 3000;
+  if (LockReadOnly()) {
+    if (!m_faxMode && m_syncMode == Mode_UserInput) {
+      if (m_receive) {
+        // Cadence for CED is single tone, but we repeat just in case
+        OnUserInputTone('Y', 3600);
+        timer = 5000;
+      }
+      else {
+        // Cadence for CNG is 500ms on 3 seconds off
+        OnUserInputTone('X', 500);
+        timer = 3000;
+      }
     }
     UnlockReadOnly();
   }
@@ -999,12 +1106,12 @@ void OpalT38Connection::OpenFaxStreams(PThread &, INT)
   OpalMediaFormat format = m_faxMode ? OpalT38 : OpalG711uLaw;
   OpalMediaType mediaType = format.GetMediaType();
 
-  PSafePtr<OpalConnection> otherParty = ownerCall.GetOtherPartyConnection(*this);
-  if (otherParty == NULL) {
-    PTRACE(1, "T38\tCannot get other party for " << mediaType << " trigger");
-  }
-  else if (!ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, 1, format)) {
+  PSafePtr<OpalConnection> other = ownerCall.GetOtherPartyConnection(*this);
+  if ( other == NULL ||
+      !ownerCall.OpenSourceMediaStreams(*other, mediaType, 1, format) ||
+      !ownerCall.OpenSourceMediaStreams(*this,  mediaType, 1, format)) {
     PTRACE(1, "T38\tMode change request to " << mediaType << " failed");
+    OnFaxCompleted(true);
   }
 
   UnlockReadWrite();
@@ -1023,7 +1130,7 @@ void OpalT38Connection::RequestFaxMode(bool toFax)
   }
 
   // definitely changing mode
-  PTRACE(1, "T38\tRequesting mode change to " << modeStr);
+  PTRACE(3, "T38\tRequesting mode change to " << modeStr);
 
   m_faxMode = toFax;
   m_faxTimer.Stop();
